/* This file is automatically generated by author/char_trait.pl */
enum { TXct_NORMAL, TXct_HTML_META = 0x01, TXct_URI_UNSAFE = 0x02 };
static const U8 char_trait[] = {
    TXct_URI_UNSAFE                /* \x00 */,
    TXct_URI_UNSAFE                /* \x01 */,
    TXct_URI_UNSAFE                /* \x02 */,
    TXct_URI_UNSAFE                /* \x03 */,
    TXct_URI_UNSAFE                /* \x04 */,
    TXct_URI_UNSAFE                /* \x05 */,
    TXct_URI_UNSAFE                /* \x06 */,
    TXct_URI_UNSAFE                /* \x07 */,
    TXct_URI_UNSAFE                /* \x08 */,
    TXct_URI_UNSAFE                /* \x09 */,
    TXct_URI_UNSAFE                /* \x0A */,
    TXct_URI_UNSAFE                /* \x0B */,
    TXct_URI_UNSAFE                /* \x0C */,
    TXct_URI_UNSAFE                /* \x0D */,
    TXct_URI_UNSAFE                /* \x0E */,
    TXct_URI_UNSAFE                /* \x0F */,
    TXct_URI_UNSAFE                /* \x10 */,
    TXct_URI_UNSAFE                /* \x11 */,
    TXct_URI_UNSAFE                /* \x12 */,
    TXct_URI_UNSAFE                /* \x13 */,
    TXct_URI_UNSAFE                /* \x14 */,
    TXct_URI_UNSAFE                /* \x15 */,
    TXct_URI_UNSAFE                /* \x16 */,
    TXct_URI_UNSAFE                /* \x17 */,
    TXct_URI_UNSAFE                /* \x18 */,
    TXct_URI_UNSAFE                /* \x19 */,
    TXct_URI_UNSAFE                /* \x1A */,
    TXct_URI_UNSAFE                /* \x1B */,
    TXct_URI_UNSAFE                /* \x1C */,
    TXct_URI_UNSAFE                /* \x1D */,
    TXct_URI_UNSAFE                /* \x1E */,
    TXct_URI_UNSAFE                /* \x1F */,
    TXct_URI_UNSAFE                /* \x20   */,
    TXct_URI_UNSAFE                /* \x21 ! */,
    TXct_URI_UNSAFE|TXct_HTML_META /* \x22 " */,
    TXct_URI_UNSAFE                /* \x23 # */,
    TXct_URI_UNSAFE                /* \x24 $ */,
    TXct_URI_UNSAFE                /* \x25 % */,
    TXct_URI_UNSAFE|TXct_HTML_META /* \x26 & */,
    TXct_URI_UNSAFE|TXct_HTML_META /* \x27 ' */,
    TXct_URI_UNSAFE                /* \x28 ( */,
    TXct_URI_UNSAFE                /* \x29 ) */,
    TXct_URI_UNSAFE                /* \x2A * */,
    TXct_URI_UNSAFE                /* \x2B + */,
    TXct_URI_UNSAFE                /* \x2C , */,
    TXct_NORMAL                    /* \x2D - */,
    TXct_NORMAL                    /* \x2E . */,
    TXct_URI_UNSAFE                /* \x2F / */,
    TXct_NORMAL                    /* \x30 0 */,
    TXct_NORMAL                    /* \x31 1 */,
    TXct_NORMAL                    /* \x32 2 */,
    TXct_NORMAL                    /* \x33 3 */,
    TXct_NORMAL                    /* \x34 4 */,
    TXct_NORMAL                    /* \x35 5 */,
    TXct_NORMAL                    /* \x36 6 */,
    TXct_NORMAL                    /* \x37 7 */,
    TXct_NORMAL                    /* \x38 8 */,
    TXct_NORMAL                    /* \x39 9 */,
    TXct_URI_UNSAFE                /* \x3A : */,
    TXct_URI_UNSAFE                /* \x3B ; */,
    TXct_URI_UNSAFE|TXct_HTML_META /* \x3C < */,
    TXct_URI_UNSAFE                /* \x3D = */,
    TXct_URI_UNSAFE|TXct_HTML_META /* \x3E > */,
    TXct_URI_UNSAFE                /* \x3F ? */,
    TXct_URI_UNSAFE                /* \x40 @ */,
    TXct_NORMAL                    /* \x41 A */,
    TXct_NORMAL                    /* \x42 B */,
    TXct_NORMAL                    /* \x43 C */,
    TXct_NORMAL                    /* \x44 D */,
    TXct_NORMAL                    /* \x45 E */,
    TXct_NORMAL                    /* \x46 F */,
    TXct_NORMAL                    /* \x47 G */,
    TXct_NORMAL                    /* \x48 H */,
    TXct_NORMAL                    /* \x49 I */,
    TXct_NORMAL                    /* \x4A J */,
    TXct_NORMAL                    /* \x4B K */,
    TXct_NORMAL                    /* \x4C L */,
    TXct_NORMAL                    /* \x4D M */,
    TXct_NORMAL                    /* \x4E N */,
    TXct_NORMAL                    /* \x4F O */,
    TXct_NORMAL                    /* \x50 P */,
    TXct_NORMAL                    /* \x51 Q */,
    TXct_NORMAL                    /* \x52 R */,
    TXct_NORMAL                    /* \x53 S */,
    TXct_NORMAL                    /* \x54 T */,
    TXct_NORMAL                    /* \x55 U */,
    TXct_NORMAL                    /* \x56 V */,
    TXct_NORMAL                    /* \x57 W */,
    TXct_NORMAL                    /* \x58 X */,
    TXct_NORMAL                    /* \x59 Y */,
    TXct_NORMAL                    /* \x5A Z */,
    TXct_URI_UNSAFE                /* \x5B [ */,
    TXct_URI_UNSAFE                /* \x5C \ */,
    TXct_URI_UNSAFE                /* \x5D ] */,
    TXct_URI_UNSAFE                /* \x5E ^ */,
    TXct_NORMAL                    /* \x5F _ */,
    TXct_URI_UNSAFE                /* \x60 ` */,
    TXct_NORMAL                    /* \x61 a */,
    TXct_NORMAL                    /* \x62 b */,
    TXct_NORMAL                    /* \x63 c */,
    TXct_NORMAL                    /* \x64 d */,
    TXct_NORMAL                    /* \x65 e */,
    TXct_NORMAL                    /* \x66 f */,
    TXct_NORMAL                    /* \x67 g */,
    TXct_NORMAL                    /* \x68 h */,
    TXct_NORMAL                    /* \x69 i */,
    TXct_NORMAL                    /* \x6A j */,
    TXct_NORMAL                    /* \x6B k */,
    TXct_NORMAL                    /* \x6C l */,
    TXct_NORMAL                    /* \x6D m */,
    TXct_NORMAL                    /* \x6E n */,
    TXct_NORMAL                    /* \x6F o */,
    TXct_NORMAL                    /* \x70 p */,
    TXct_NORMAL                    /* \x71 q */,
    TXct_NORMAL                    /* \x72 r */,
    TXct_NORMAL                    /* \x73 s */,
    TXct_NORMAL                    /* \x74 t */,
    TXct_NORMAL                    /* \x75 u */,
    TXct_NORMAL                    /* \x76 v */,
    TXct_NORMAL                    /* \x77 w */,
    TXct_NORMAL                    /* \x78 x */,
    TXct_NORMAL                    /* \x79 y */,
    TXct_NORMAL                    /* \x7A z */,
    TXct_URI_UNSAFE                /* \x7B { */,
    TXct_URI_UNSAFE                /* \x7C | */,
    TXct_URI_UNSAFE                /* \x7D } */,
    TXct_NORMAL                    /* \x7E ~ */,
    TXct_URI_UNSAFE                /* \x7F */,
    TXct_URI_UNSAFE                /* \x80 */,
    TXct_URI_UNSAFE                /* \x81 */,
    TXct_URI_UNSAFE                /* \x82 */,
    TXct_URI_UNSAFE                /* \x83 */,
    TXct_URI_UNSAFE                /* \x84 */,
    TXct_URI_UNSAFE                /* \x85 */,
    TXct_URI_UNSAFE                /* \x86 */,
    TXct_URI_UNSAFE                /* \x87 */,
    TXct_URI_UNSAFE                /* \x88 */,
    TXct_URI_UNSAFE                /* \x89 */,
    TXct_URI_UNSAFE                /* \x8A */,
    TXct_URI_UNSAFE                /* \x8B */,
    TXct_URI_UNSAFE                /* \x8C */,
    TXct_URI_UNSAFE                /* \x8D */,
    TXct_URI_UNSAFE                /* \x8E */,
    TXct_URI_UNSAFE                /* \x8F */,
    TXct_URI_UNSAFE                /* \x90 */,
    TXct_URI_UNSAFE                /* \x91 */,
    TXct_URI_UNSAFE                /* \x92 */,
    TXct_URI_UNSAFE                /* \x93 */,
    TXct_URI_UNSAFE                /* \x94 */,
    TXct_URI_UNSAFE                /* \x95 */,
    TXct_URI_UNSAFE                /* \x96 */,
    TXct_URI_UNSAFE                /* \x97 */,
    TXct_URI_UNSAFE                /* \x98 */,
    TXct_URI_UNSAFE                /* \x99 */,
    TXct_URI_UNSAFE                /* \x9A */,
    TXct_URI_UNSAFE                /* \x9B */,
    TXct_URI_UNSAFE                /* \x9C */,
    TXct_URI_UNSAFE                /* \x9D */,
    TXct_URI_UNSAFE                /* \x9E */,
    TXct_URI_UNSAFE                /* \x9F */,
    TXct_URI_UNSAFE                /* \xA0 */,
    TXct_URI_UNSAFE                /* \xA1 */,
    TXct_URI_UNSAFE                /* \xA2 */,
    TXct_URI_UNSAFE                /* \xA3 */,
    TXct_URI_UNSAFE                /* \xA4 */,
    TXct_URI_UNSAFE                /* \xA5 */,
    TXct_URI_UNSAFE                /* \xA6 */,
    TXct_URI_UNSAFE                /* \xA7 */,
    TXct_URI_UNSAFE                /* \xA8 */,
    TXct_URI_UNSAFE                /* \xA9 */,
    TXct_URI_UNSAFE                /* \xAA */,
    TXct_URI_UNSAFE                /* \xAB */,
    TXct_URI_UNSAFE                /* \xAC */,
    TXct_URI_UNSAFE                /* \xAD */,
    TXct_URI_UNSAFE                /* \xAE */,
    TXct_URI_UNSAFE                /* \xAF */,
    TXct_URI_UNSAFE                /* \xB0 */,
    TXct_URI_UNSAFE                /* \xB1 */,
    TXct_URI_UNSAFE                /* \xB2 */,
    TXct_URI_UNSAFE                /* \xB3 */,
    TXct_URI_UNSAFE                /* \xB4 */,
    TXct_URI_UNSAFE                /* \xB5 */,
    TXct_URI_UNSAFE                /* \xB6 */,
    TXct_URI_UNSAFE                /* \xB7 */,
    TXct_URI_UNSAFE                /* \xB8 */,
    TXct_URI_UNSAFE                /* \xB9 */,
    TXct_URI_UNSAFE                /* \xBA */,
    TXct_URI_UNSAFE                /* \xBB */,
    TXct_URI_UNSAFE                /* \xBC */,
    TXct_URI_UNSAFE                /* \xBD */,
    TXct_URI_UNSAFE                /* \xBE */,
    TXct_URI_UNSAFE                /* \xBF */,
    TXct_URI_UNSAFE                /* \xC0 */,
    TXct_URI_UNSAFE                /* \xC1 */,
    TXct_URI_UNSAFE                /* \xC2 */,
    TXct_URI_UNSAFE                /* \xC3 */,
    TXct_URI_UNSAFE                /* \xC4 */,
    TXct_URI_UNSAFE                /* \xC5 */,
    TXct_URI_UNSAFE                /* \xC6 */,
    TXct_URI_UNSAFE                /* \xC7 */,
    TXct_URI_UNSAFE                /* \xC8 */,
    TXct_URI_UNSAFE                /* \xC9 */,
    TXct_URI_UNSAFE                /* \xCA */,
    TXct_URI_UNSAFE                /* \xCB */,
    TXct_URI_UNSAFE                /* \xCC */,
    TXct_URI_UNSAFE                /* \xCD */,
    TXct_URI_UNSAFE                /* \xCE */,
    TXct_URI_UNSAFE                /* \xCF */,
    TXct_URI_UNSAFE                /* \xD0 */,
    TXct_URI_UNSAFE                /* \xD1 */,
    TXct_URI_UNSAFE                /* \xD2 */,
    TXct_URI_UNSAFE                /* \xD3 */,
    TXct_URI_UNSAFE                /* \xD4 */,
    TXct_URI_UNSAFE                /* \xD5 */,
    TXct_URI_UNSAFE                /* \xD6 */,
    TXct_URI_UNSAFE                /* \xD7 */,
    TXct_URI_UNSAFE                /* \xD8 */,
    TXct_URI_UNSAFE                /* \xD9 */,
    TXct_URI_UNSAFE                /* \xDA */,
    TXct_URI_UNSAFE                /* \xDB */,
    TXct_URI_UNSAFE                /* \xDC */,
    TXct_URI_UNSAFE                /* \xDD */,
    TXct_URI_UNSAFE                /* \xDE */,
    TXct_URI_UNSAFE                /* \xDF */,
    TXct_URI_UNSAFE                /* \xE0 */,
    TXct_URI_UNSAFE                /* \xE1 */,
    TXct_URI_UNSAFE                /* \xE2 */,
    TXct_URI_UNSAFE                /* \xE3 */,
    TXct_URI_UNSAFE                /* \xE4 */,
    TXct_URI_UNSAFE                /* \xE5 */,
    TXct_URI_UNSAFE                /* \xE6 */,
    TXct_URI_UNSAFE                /* \xE7 */,
    TXct_URI_UNSAFE                /* \xE8 */,
    TXct_URI_UNSAFE                /* \xE9 */,
    TXct_URI_UNSAFE                /* \xEA */,
    TXct_URI_UNSAFE                /* \xEB */,
    TXct_URI_UNSAFE                /* \xEC */,
    TXct_URI_UNSAFE                /* \xED */,
    TXct_URI_UNSAFE                /* \xEE */,
    TXct_URI_UNSAFE                /* \xEF */,
    TXct_URI_UNSAFE                /* \xF0 */,
    TXct_URI_UNSAFE                /* \xF1 */,
    TXct_URI_UNSAFE                /* \xF2 */,
    TXct_URI_UNSAFE                /* \xF3 */,
    TXct_URI_UNSAFE                /* \xF4 */,
    TXct_URI_UNSAFE                /* \xF5 */,
    TXct_URI_UNSAFE                /* \xF6 */,
    TXct_URI_UNSAFE                /* \xF7 */,
    TXct_URI_UNSAFE                /* \xF8 */,
    TXct_URI_UNSAFE                /* \xF9 */,
    TXct_URI_UNSAFE                /* \xFA */,
    TXct_URI_UNSAFE                /* \xFB */,
    TXct_URI_UNSAFE                /* \xFC */,
    TXct_URI_UNSAFE                /* \xFD */,
    TXct_URI_UNSAFE                /* \xFE */,
    TXct_URI_UNSAFE                /* \xFF */,
}; /* char_trait */
